;;;; message-digest-test.scm

;; Issues
;;
;; - Needs many more tests, especially the entire input-port & procedure source stuff.

(use test)
(use message-digest message-digest-port)
(use files lolevel srfi-4)
(use setup-api)

;;

(define (ashexstr s)
	(apply string-append
		(map
			(lambda (c) (number->string (char->integer c) 16))
			(string->list s))) )

(define simple-src "ab cd")
(define simple-res (ashexstr simple-src))

(define-constant DIGEST-LENGTH 5)
(define-constant CONTEXT-SIZE 10)
(define-constant BLOCK-LENGTH 64)

(define-constant BLOCK-LENGTH-DEFAULT 4)

(define SHORT-TEST-FILE-NAME "alpha.txt")
(define SHORT-TEST-FILE-LENGTH 26)

(define just-once
  (let ((x #t))
    (lambda ()
      (let (
        (res (and x simple-src)) )
        (set! x #f)
        res ) ) ) )

;FIXME add (mock-*-primitive ...) that wraps the supplied phase procedures

;;

(test-begin "Message Digest")

(test 'hex-string (message-digest-result-form 'hex))

;Tests defaults
(test-group "Chunk Read (port)"
  (let (
    (siz (message-digest-chunk-size))
    (in (open-input-file SHORT-TEST-FILE-NAME)) )
    (let* (
      (rdr ((message-digest-chunk-port-read-maker) in))
      (res (rdr)) )
      (test-assert "First chunk type" (blob? res))
      (test "First chunk size" SHORT-TEST-FILE-LENGTH (blob-size res))
      (test-assert "No more chunk" (not (rdr))) )
    (close-input-port in) )
)

(test-group "Make Primitive"

	(define the-ctx #f)

  (define (init ctx)
    (set! the-ctx ctx) )

  (define (update ctx bytes count)
    (assert (eq? ctx the-ctx))
    (assert (not (not bytes)))
    (assert (< 0 count))
    (void) )

  (define (final ctx result)
    (assert (eq? ctx the-ctx))
    (assert (not (not result)))
    (void) )

  (let (
    (mdp (make-message-digest-primitive CONTEXT-SIZE DIGEST-LENGTH init update final)) )
    (test-assert (message-digest-primitive? mdp))
    (test CONTEXT-SIZE (message-digest-primitive-context-info mdp))
    (test DIGEST-LENGTH (message-digest-primitive-digest-length mdp))
    (test init (message-digest-primitive-init mdp))
    (test update (message-digest-primitive-update mdp))
    (test final (message-digest-primitive-final mdp))
    (test BLOCK-LENGTH-DEFAULT (message-digest-primitive-block-length mdp))
    (test-assert (symbol? (message-digest-primitive-name mdp))) )

  ;;don't bother testing the non-optional arguments again

  (let (
    (mdp
      (make-message-digest-primitive CONTEXT-SIZE DIGEST-LENGTH init update final 'foo)) )
    (test BLOCK-LENGTH-DEFAULT (message-digest-primitive-block-length mdp))
    (test 'foo (message-digest-primitive-name mdp)) )

  (let (
    (mdp
      (make-message-digest-primitive CONTEXT-SIZE DIGEST-LENGTH init update final BLOCK-LENGTH)) )
    (test BLOCK-LENGTH (message-digest-primitive-block-length mdp))
    (test-assert (symbol? (message-digest-primitive-name mdp))) )

  (let (
    (mdp
      (make-message-digest-primitive CONTEXT-SIZE DIGEST-LENGTH init update final BLOCK-LENGTH 'foo)) )
    (test BLOCK-LENGTH (message-digest-primitive-block-length mdp))
    (test 'foo (message-digest-primitive-name mdp)) )
)

;These also test the update-string proc
(test-group "Proper Phase Arguments (Def Alloc)"

	(define the-ctx #f)

  (define (init ctx)
    ;(printf "  Init Ctx: ~S~%" ctx)
    (set! the-ctx ctx)
    (assert (pointer? ctx)) )

  (define (update ctx bytes count)
  	;(printf "Update Ctx: ~S Bytes: ~S Count: ~S~%" ctx bytes count)
    (assert (eq? ctx the-ctx))
    (assert (not (not bytes)))
    (assert (< 0 count))
    (assert (<= count CONTEXT-SIZE))  ; So no mem overflow
    (assert (pointer? ctx))
    (assert (blob? bytes))
    (assert (<= count (blob-size bytes)))
    (move-memory! bytes ctx count) )

  (define (final ctx result)
    ;(printf " Final Ctx: ~S Result: ~S~%" ctx result)
    (assert (eq? ctx the-ctx))
    (assert (not (not result)))
    (assert (pointer? ctx))
    (assert (or (blob? result) (string? result)))
    ; So no mem overflow
    (assert (<= DIGEST-LENGTH (if (blob? result) (blob-size result) (string-length result))))
    (move-memory! ctx result DIGEST-LENGTH) )

  (let (
    (mdp (make-message-digest-primitive CONTEXT-SIZE DIGEST-LENGTH init update final)) )
    (let (
      (md (initialize-message-digest mdp)) )
      (test-assert (message-digest? md))
      (test-assert (message-digest-update-string md simple-src))
      (test simple-res (finalize-message-digest md)) )
    (let (
      (md (initialize-message-digest mdp))
      (blb (make-blob 100)) )
      (message-digest-update-string md simple-src)
      (test-assert (finalize-message-digest! md blb))
      (print "result = " blb) )
    (let (
      (md (initialize-message-digest mdp))
      (str (make-string 100 #\space)) )
      (message-digest-update-string md simple-src)
      (test-assert (finalize-message-digest! md str))
      (print "result = " #\" str #\") )
    (let (
      (md (initialize-message-digest mdp))
      (vec (make-u8vector 100 0)) )
      (message-digest-update-string md simple-src)
      (test-assert (finalize-message-digest! md vec))
      (print "result = " vec) ) )
)

(test-group "Proper Phase Arguments (Own Alloc)"

	(define the-ctx #f)

  (define (make-context)
    (make-blob CONTEXT-SIZE) )

  (define (init ctx)
    ;(printf "  Init Ctx: ~S~%" ctx)
    (set! the-ctx ctx)
    (assert (blob? ctx)) )

  (define (update ctx bytes count)
    ;(printf "Update Ctx: ~S Bytes: ~S Count: ~S~%" ctx bytes count) (flush-output)
    (assert (eq? ctx the-ctx))
    (assert (not (not bytes)))
    (assert (< 0 count))
    (assert (<= count CONTEXT-SIZE))  ; So no mem overflow
    (assert (blob? ctx))
    (assert (blob? bytes))
    (assert (<= count (blob-size bytes)))
    (move-memory! bytes ctx count) )

  (define (final ctx result)
    ;(printf " Final Result Size: ~S Ctx: ~S Result: ~S~%" (blob-size result) ctx result) (flush-output)
    (assert (eq? ctx the-ctx))
    (assert (not (not result)))
    (assert (blob? ctx))
    (assert (blob? result))
    (assert (<= (blob-size result) DIGEST-LENGTH))  ; So no mem overflow
    (move-memory! ctx result DIGEST-LENGTH) )

  (let* (
    (mdp (make-message-digest-primitive make-context DIGEST-LENGTH init update final))
    (md (initialize-message-digest mdp)) )
    (test-assert (message-digest? md))
    (test-assert (message-digest-update-string md simple-src))
    (test simple-res (finalize-message-digest md)) )
)

(let ()

	(define the-ctx #f)

  (define (make-context)
    ;Init to 0 necessary since DIGEST-LENGTH is possibly > than
    ;the input size! (Actually just needs to be a known value,
    ;`(integer->char #xff)' would work as well.)
    (string->blob (make-string CONTEXT-SIZE #\nul)) )

  (define (init ctx)
    (set! the-ctx ctx) )

  (define (update ctx bytes count)
    ;(printf "Update Ctx: ~S Bytes: ~S Count: ~S~%" ctx bytes count) (flush-output)
    (assert (eq? ctx the-ctx))
    (assert (not (not bytes)))
    (assert (< 0 count))
    (assert (<= count CONTEXT-SIZE))  ; So no mem overflow
    (assert (blob? ctx))
    (move-memory! bytes ctx count) )

  (define (final ctx result)
    ;(printf " Final Result Size: ~S Ctx: ~S Result: ~S~%" (blob-size result) ctx result) (flush-output)
    (assert (eq? ctx the-ctx))
    (assert (not (not result)))
    (assert (blob? ctx))
    (assert (<= (blob-size result) DIGEST-LENGTH))  ; So no mem overflow
    (move-memory! ctx result DIGEST-LENGTH) )

  (define mdp (make-message-digest-primitive make-context DIGEST-LENGTH init update final))

  (test-group "u8vector Source"
    (let (
      (md (initialize-message-digest mdp)) )
      (test-assert (message-digest-update-u8vector md (u8vector 1 2 3 4 5)))
      (test "0102030405" (finalize-message-digest md)) )
  )

  (test-group "u8 Source"
    (let (
      (md (initialize-message-digest mdp)) )
      (test-assert (message-digest-update-u8 md #xA2))
      (test "a200000000" (finalize-message-digest md)) )
  )

  (test-group "u16-le Source"
    (let (
      (md (initialize-message-digest mdp)) )
      (test-assert (message-digest-update-u16-le md #xA2B2))
      (test "b2a2000000" (finalize-message-digest md)) )
  )

  (test-group "u32-be Source"
    (let (
      (md (initialize-message-digest mdp)) )
      (test-assert (message-digest-update-u32-be md 1073741823))
      (test "3fffffff00" (finalize-message-digest md)) )
  )

  (when (version>=? (chicken-version) "4.6.4")
	  (test-group "u32-be Source"
			(let (
        (md (initialize-message-digest mdp)) )
				(test-assert (message-digest-update-u32-be md #xA2B2C2D2))
				(test "a2b2c2d200" (finalize-message-digest md)) )
		) )

  (when (version>=? (chicken-version) "4.8.1")
		(test-group "u64-be Source"
			(let (
        (md (initialize-message-digest mdp)) )
				(test-assert (message-digest-update-u64-be md #xAB54A98CEB1F0AD2))
				(test
				  (substring "ab54a98ceb1f0ad2" 0 (fx* 2 (fxmin 8 DIGEST-LENGTH)))
				  (finalize-message-digest md)) )
		) )

  (test-group "char-u8 Source"
    (let (
      (md (initialize-message-digest mdp)) )
      (test-assert (message-digest-update-char-u8 md #\space))
      (test "2000000000" (finalize-message-digest md)) )
  )

  (test-group "char-be Source"
    (let (
      (md (initialize-message-digest mdp)) )
      (test-assert (message-digest-update-char-be md #\u0003BB))
      (test "000003bb00" (finalize-message-digest md)) )
  )

  (test-group "char-le Source"
    (let (
      (md (initialize-message-digest mdp)) )
      (test-assert (message-digest-update-char-le md #\u0003BB))
      (test "bb03000000" (finalize-message-digest md)) )
  )

  (test-group "Procedure Source"
    (let (
      (md (initialize-message-digest mdp)) )
      (test-assert (message-digest-update-procedure md just-once))
      (test simple-res (finalize-message-digest md)) )
  )

  (test-group "Port"
    (let (
      (port (open-output-digest mdp)) )
      (test-assert (output-port? port))
      (display simple-src port) ;cannot be readable!
      (test simple-res (get-output-digest port))
      (test-assert (port-closed? port)) )
  )
)

#; ;REMOVED
(begin
	(use message-digest-old)

	(define the-ctx #f)

  (define (make-context)
    (string->blob (make-string CONTEXT-SIZE #\nul)) )

  (define (init ctx)
    (set! the-ctx ctx) )

  (define (update ctx bytes count)
    (assert (eq? ctx the-ctx))
    (assert (not (not bytes)))
    (assert (< 0 count))
    (assert (<= count CONTEXT-SIZE))  ; So no mem overflow
    (assert (blob? ctx))
    (move-memory! bytes ctx count) )

  (define (final ctx result)
    (assert (eq? ctx the-ctx))
    (assert (not (not result)))
    (assert (blob? ctx))
    (assert (<= (blob-size result)  DIGEST-LENGTH))  ; So no mem overflow
    (move-memory! ctx result DIGEST-LENGTH) )

  (test-group "Primitive Apply (DEPRECATED)"
    (let* (
      (mdp
        (make-message-digest-primitive CONTEXT-SIZE DIGEST-LENGTH init update final))
      (res (message-digest-primitive-apply mdp simple-src)) )
      (test-assert (string? res))
      (test simple-res (byte-string->hexadecimal res)) )
  )

  (test-group "Make (DEPRECATED)"
    (test simple-src (make-binary-message-digest simple-src make-context DIGEST-LENGTH init update final))
    (test simple-res (make-message-digest simple-src make-context DIGEST-LENGTH init update final))
  )
)

;
(test-group "Chunk Read (fileno)"

	(define the-ctx #f)

  (define (init ctx)
    ;(printf "  Init Ctx: ~S~%" ctx)
    (assert (pointer? ctx))
    (set! the-ctx ctx) )

  (define (update ctx bytes count)
  	;(printf "Update Ctx: ~S Count: ~S Bytes: ~S~%" ctx count bytes)
    (assert (eq? ctx the-ctx))
    (assert (not (not bytes)))
    (assert (< 0 count))
    (assert (<= count CONTEXT-SIZE))
    (assert (pointer? ctx))
    (move-memory! bytes ctx count) )

  (define (raw-update ctx bytes count)
  	;(printf "Raw-Update Ctx: ~S Count: ~S Bytes: ~S~%" ctx count bytes)
    (assert (eq? ctx the-ctx))
    (assert (not (not bytes)))
    (assert (< 0 count))
    (assert (= SHORT-TEST-FILE-LENGTH count))
    (assert (pointer? ctx))
    (move-memory! bytes ctx (min CONTEXT-SIZE count)) )

  (define (final ctx result)
    ;(printf " Final Ctx: ~S Length: ~S Result: ~S~%" ctx DIGEST-LENGTH result)
    (assert (eq? ctx the-ctx))
    (assert (not (not result)))
    (assert (pointer? ctx))
    ;(assert (<= 0 DIGEST-LENGTH))
    (move-memory! ctx result DIGEST-LENGTH) )

  (let (
    (mdp
      (make-message-digest-primitive CONTEXT-SIZE DIGEST-LENGTH init update final raw-update)) )
    (test "6162636465" (message-digest-file mdp SHORT-TEST-FILE-NAME 'hex-string)) )
)

(test-end)

(test-exit)
